package com.ruoyi.ss.account.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.valid.bank.BankCardInfo;
import com.ruoyi.common.valid.bank.BankValidUtils;
import com.ruoyi.ss.account.domain.SmAccount;
import com.ruoyi.ss.account.domain.SmAccountBO;
import com.ruoyi.ss.account.domain.SmAccountQuery;
import com.ruoyi.ss.account.domain.SmAccountVo;
import com.ruoyi.ss.account.domain.enums.AccountType;
import com.ruoyi.ss.account.mapper.SmAccountMapper;
import com.ruoyi.ss.account.service.ISmAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 用户账户Service业务层处理
 *
 * @author ruoyi
 * @date 2024-03-26
 */
@Service
public class SmAccountServiceImpl implements ISmAccountService
{
    @Autowired
    private SmAccountMapper smAccountMapper;

    /**
     * 查询用户账户
     *
     * @param accountId 用户账户主键
     * @return 用户账户
     */
    @Override
    public SmAccountVo selectSmAccountByAccountId(Long accountId)
    {
        SmAccountVo account = smAccountMapper.selectSmAccountByAccountId(accountId);
        account.desensitization();
        return account;
    }

    /**
     * 查询用户账户列表
     *
     * @param dto 用户账户
     * @return 用户账户
     */
    @Override
    public List<SmAccountVo> selectSmAccountList(SmAccountQuery dto)
    {
        List<SmAccountVo> list = smAccountMapper.selectSmAccountList(dto);
        list.forEach(SmAccount::desensitization);
        return list;
    }

    /**
     * 新增用户账户
     *
     * @param smAccount 用户账户
     * @return 结果
     */
    @Override
    public int insertSmAccount(SmAccount smAccount)
    {
        smAccount.setCreateTime(DateUtils.getNowDate());
        return smAccountMapper.insertSmAccount(smAccount);
    }

    /**
     * 修改用户账户
     *
     * @param smAccount 用户账户
     * @return 结果
     */
    @Override
    public int updateSmAccount(SmAccount smAccount)
    {
        return smAccountMapper.updateSmAccount(smAccount);
    }

    /**
     * 批量删除用户账户
     *
     * @param accountIds 需要删除的用户账户主键
     * @return 结果
     */
    @Override
    public int deleteSmAccountByAccountIds(Long[] accountIds)
    {
        return smAccountMapper.deleteSmAccountByAccountIds(accountIds);
    }

    /**
     * 删除用户账户信息
     *
     * @param accountId 用户账户主键
     * @return 结果
     */
    @Override
    public int deleteSmAccountByAccountId(Long accountId)
    {
        return smAccountMapper.deleteSmAccountByAccountId(accountId);
    }

    @Override
    public boolean addAccount(SmAccountBO data) {
        // 查重
        ServiceUtil.assertion(isRepeat(data), "已绑定的账户不可重复绑定");

        if (AccountType.BANK_CARD.getType().equals(data.getAccountType())) {
            // 银行卡三要素校验，并获取卡面信息
            BankCardInfo bankCardInfo = BankValidUtils.valid3Bank(data.getAccountNo(), data.getName(), data.getIdCard());
            data.setCardInfo(bankCardInfo);
        }

        // 保存数据
        return this.insertSmAccount(data) == 1;
    }

    @Override
    public Map<String, List<SmAccountVo>> selectSmAccountGroup(SmAccountQuery dto) {
        List<SmAccountVo> list = this.selectSmAccountList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        Map<String, List<SmAccountVo>> map = new HashMap<>();
        map.put(AccountType.BANK_CARD.name(), list.stream()
                .filter(item -> Objects.equals(AccountType.BANK_CARD.getType(), item.getAccountType())).collect(Collectors.toList()));
        map.put(AccountType.WECHAT.name(), list.stream()
                .filter(item -> Objects.equals(AccountType.WECHAT.getType(), item.getAccountType())).collect(Collectors.toList()));
        map.put(AccountType.ALIPAY.name(), list.stream()
                .filter(item -> Objects.equals(AccountType.ALIPAY.getType(), item.getAccountType())).collect(Collectors.toList()));
        return map;
    }

    /**
     * 判断用户是否有这个账号
     * @param userId 用户id
     * @param accountNo 账号
     * @param accountType 账号类型
     */
    @Override
    public boolean validateUserHasAccount(Long userId, String accountNo, AccountType accountType) {
        if (userId == null || accountNo == null) {
            return false;
        }
        SmAccountVo account = smAccountMapper.selectByAccountNo(accountNo, accountType.getType());
        return account != null && account.getUserId() != null && account.getUserId().equals(userId);
    }

    @Override
    public SmAccountVo selectUserWxAccount(Long userId) {
        if (userId == null) {
            return null;
        }
        SmAccountQuery dto = new SmAccountQuery();
        dto.setUserId(userId);
        dto.setAccountType(AccountType.WECHAT.getType());
        List<SmAccountVo> list = smAccountMapper.selectSmAccountList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    public SmAccountVo selectUseAliAccount(Long userId) {
        if (userId == null) {
            return null;
        }
        SmAccountQuery dto = new SmAccountQuery();
        dto.setUserId(userId);
        dto.setAccountType(AccountType.ALIPAY.getType());
        List<SmAccountVo> list = smAccountMapper.selectSmAccountList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    public boolean setDefault(Long userId, Long accountId) {
        SmAccountVo account = smAccountMapper.selectSmAccountByAccountId(accountId);
        ServiceUtil.assertion(account == null, "账户不存在");
        ServiceUtil.assertion(!Objects.equals(account.getUserId(), userId), "该账户不是本用户的账户");

        int updateCount = smAccountMapper.setDefault(userId, accountId);
        ServiceUtil.assertion(updateCount == 0, "设置默认账户失败");
        return true;
    }

    private boolean isRepeat(SmAccountBO data) {
        SmAccountQuery dto = new SmAccountQuery();
        dto.setAccountNo(data.getAccountNo());
        dto.setUserId(data.getUserId());
        List<SmAccountVo> repeats = smAccountMapper.selectSmAccountList(dto);
        if (!CollectionUtils.isEmpty(repeats)) {
            for (SmAccountVo repeat : repeats) {
                if (!Objects.equals(repeat.getAccountId(), data.getAccountId())) {
                    return true;
                }
            }
        }
        return false;
    }
}
