package com.ruoyi.ss.suit.service.impl;

import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.suit.domain.Suit;
import com.ruoyi.ss.suit.domain.SuitQuery;
import com.ruoyi.ss.suit.domain.SuitVo;
import com.ruoyi.ss.suit.domain.dto.SuitCopyDTO;
import com.ruoyi.ss.suit.mapper.SuitMapper;
import com.ruoyi.ss.suit.service.SuitConverter;
import com.ruoyi.ss.suit.service.SuitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 套餐Service业务层处理
 *
 * @author ruoyi
 * @date 2024-04-25
 */
@Service
public class SuitServiceImpl implements SuitService
{
    @Autowired
    private SuitMapper suitMapper;

    @Autowired
    private SuitConverter suitConverter;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询套餐
     *
     * @param suitId 套餐主键
     * @return 套餐
     */
    @Override
    public SuitVo selectSuitBySuitId(Long suitId)
    {
        return suitMapper.selectSuitBySuitId(suitId);
    }

    /**
     * 查询套餐列表
     *
     * @param suit 套餐
     * @return 套餐
     */
    @Override
    public List<SuitVo> selectSuitList(SuitQuery suit)
    {
        return suitMapper.selectSuitList(suit);
    }

    /**
     * 新增套餐
     *
     * @param suit 套餐
     * @return 结果
     */
    @Override
    public int insertSuit(Suit suit)
    {
        this.preInsert(suit);
        return suitMapper.insertSuit(suit);
    }

    @Override
    public int batchInsert(List<Suit> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return 0;
        }
        for (Suit suit : list) {
            this.preInsert(suit);
        }
        return suitMapper.batchInsert(list);
    }

    /**
     * 插入前操作
     */
    private void preInsert(Suit suit) {
        suit.setCreateTime(DateUtils.getNowDate());
    }


    /**
     * 修改套餐
     *
     * @param suit 套餐
     * @return 结果
     */
    @Override
    public int updateSuit(Suit suit) {
        SuitVo old = selectSuitBySuitId(suit.getSuitId());
        ServiceUtil.assertion(old == null, "待修改的数据不存在");

        suit.setUpdateTime(DateUtils.getNowDate());
        Integer result = transactionTemplate.execute(status -> {
            // 清除本数据的来源ID
            suitMapper.clearSource(suit.getSuitId());

            // 更新数据
            int update = suitMapper.updateSuit(suit);
            ServiceUtil.assertion(update != 1, "更新数据失败");

            // 更新来源为本套餐的数据
            suitMapper.updateSuitBySource(suit);

            return update;
        });

        return result == null ? 0 : result;
    }

    /**
     * 批量删除套餐
     *
     * @param suitIds 需要删除的套餐主键
     * @return 结果
     */
    @Override
    public int deleteSuitBySuitIds(Long[] suitIds)
    {
        return suitMapper.deleteSuitBySuitIds(suitIds);
    }

    /**
     * 删除套餐信息
     *
     * @param suitId 套餐主键
     * @return 结果
     */
    @Override
    public int deleteSuitBySuitId(Long suitId)
    {
        return suitMapper.deleteSuitBySuitId(suitId);
    }

    /**
     * 批量逻辑删除
     *
     * @param suitIds
     * @return
     */
    @Override
    public int logicDel(List<Long> suitIds) {
        return suitMapper.logicDel(suitIds);
    }

    /**
     * 通用查询映射方法
     * @param query 查询条件
     * @param keyMapper 映射方法,需要保证唯一性
     */
    @Override
    public <K> Map<K, SuitVo> selectMap(SuitQuery query, Function<? super SuitVo, ? extends K> keyMapper) {
        List<SuitVo> list = selectSuitList(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, item -> item));
    }

    /**
     * 通用分组方法
     *
     * @param query     查询条件
     * @param keyMapper 分组映射方法，需要保证唯一性
     */
    @Override
    public <K> Map<K, List<SuitVo>> selectGroup(SuitQuery query, Function<? super SuitVo, ? extends K> keyMapper) {
        List<SuitVo> list = selectSuitList(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.groupingBy(keyMapper));
    }

    /**
     * 根据设备查询套餐列表
     *
     * @param deviceId
     * @return
     */
    @Override
    public List<SuitVo> selectByDeviceId(Long deviceId) {
        if (deviceId == null) {
            return Collections.emptyList();
        }
        SuitQuery query = new SuitQuery();
        query.setDeviceId(deviceId);
        return this.selectSuitList(query);
    }

    /**
     * 查询套餐数量
     *
     * @param query
     * @return
     */
    @Override
    public int selectCount(SuitQuery query) {
        return suitMapper.selectCount(query);
    }

    @Override
    public int logicDelByDeviceId(Long deviceId) {
        if (deviceId == null) {
            return 0;
        }
        return suitMapper.logicDelByDeviceId(deviceId);
    }

    @Override
    public SuitVo selectOne(SuitQuery query) {
        return suitMapper.selectOne(query);
    }

    @Override
    public int copy(SuitCopyDTO dto) {
        ServiceUtil.assertion(dto == null, "参数不允许为空");
        List<Suit> suitList = suitConverter.toPoList(dto);
        return this.batchInsert(suitList);
    }
}
