package com.ruoyi.ss.wxPay.service;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.config.WxPayConfig;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.pay.wx.domain.NotifyEventType;
import com.ruoyi.common.pay.wx.domain.TransferBatchStatus;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillPayType;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.ss.wxPay.domain.RefundAble;
import com.ruoyi.ss.wxPay.domain.enums.TransferScene;
import com.ruoyi.ss.wxPayNotify.domain.SmWxPayNotify;
import com.ruoyi.ss.wxPayNotify.mapper.SmWxPayNotifyMapper;
import com.wechat.pay.java.core.notification.Notification;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.jsapi.JsapiService;
import com.wechat.pay.java.service.payments.jsapi.JsapiServiceExtension;
import com.wechat.pay.java.service.payments.jsapi.model.*;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.RefundService;
import com.wechat.pay.java.service.refund.model.CreateRequest;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.transferbatch.TransferBatchService;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferRequest;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferResponse;
import com.wechat.pay.java.service.transferbatch.model.TransferBatchGet;
import com.wechat.pay.java.service.transferbatch.model.TransferDetailInput;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 微信支付服务
 * @author 辉
 * 2024/3/11
 */
@Service
@Slf4j
public class WxPayService implements IWxPayService {

    @Autowired
    private JsapiService jsapiService;

    @Autowired
    private JsapiServiceExtension jsapiServiceExtension;

    @Autowired
    private WxPayConfig wxPayConfig;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private ISmUserService smUserService;

    @Autowired
    private NotificationParser notificationParser;

    @Autowired
    private SmWxPayNotifyMapper smWxPayNotifyMapper;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private RefundService refundService;

    @Autowired
    private TransferBatchService transferBatchService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    private static final String CNY = "CNY";

    @Override
    @Transactional
    public PrepayWithRequestPaymentResponse prepayWithRequestPayment(String billNo) {
        try {
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.PREPAY.getKey() + ":" + billNo), "当前订单正在支付，请刷新后重试");

            TransactionBill bill = transactionBillService.selectSmTransactionBillByBillNo(billNo);
            ServiceUtil.assertion(bill == null, "未查询到相关订单");
            ServiceUtil.assertion(!Objects.equals(bill.getChannelId(), TransactionBillPayType.WECHAT.getType()), "该订单不是微信支付订单");
            ServiceUtil.assertion(!TransactionBillStatus.UNPAID.getStatus().equals(bill.getStatus()), "只能支付未支付的订单");
            ServiceUtil.assertion(!TransactionBillType.RECHARGE.getType().equals(bill.getType()), "只能支付充值订单");

            SmUserVo user = smUserService.selectSmUserByUserId(bill.getUserId());
            ServiceUtil.assertion(user == null, "用户不存在");
            ServiceUtil.assertion(user.getWxOpenId() == null, "用户微信openId为空");

            // 获取JSAPI所需参数
            PrepayRequest request = new PrepayRequest();
            request.setAmount(getAmount(bill.getMoney()));
            request.setOutTradeNo(billNo);
            request.setAppid(wxPayConfig.getAppId());
            request.setMchid(wxPayConfig.getMerchantId());
            request.setDescription(getDescription(bill));
            request.setNotifyUrl(wxPayConfig.getNotifyUrl());
            request.setPayer(getPayer(user.getWxOpenId()));

            PrepayWithRequestPaymentResponse res = jsapiServiceExtension.prepayWithRequestPayment(request);

            // 支付中
            try {
                // 支付中
                transactionBillService.paying(bill.getBillId());

                // 每隔20秒查询支付结果，直到过期，首次10秒查询
                scheduledExecutorService.schedule(() -> {
                    transactionBillService.refreshPayResultBeforeExpire(bill.getBillNo(), 20, TimeUnit.SECONDS);
                }, 10, TimeUnit.SECONDS);

            } catch (Exception e) {
                this.closeOrder(bill.getBillNo());
                throw e;
            }

            return res;
        } finally {
            redisLock.unlock(RedisLockKey.PREPAY.getKey() + ":" + billNo);
        }
    }

    /**
     * 关闭支付订单
     * @param billNo 平台订单编号
     */
    @Override
    public void closeOrder(String billNo) {
        CloseOrderRequest request = new CloseOrderRequest();
        request.setMchid(wxPayConfig.getMerchantId());
        request.setOutTradeNo(billNo);
        jsapiService.closeOrder(request);
    }

    @Override
    public Transaction queryOrderById(String prePayId) {
        QueryOrderByIdRequest request = new QueryOrderByIdRequest();
        request.setMchid(wxPayConfig.getMerchantId());
        request.setTransactionId(prePayId);
        return jsapiService.queryOrderById(request);
    }

    @Override
    public Transaction queryOrderByOutTradeNo(String billNo) {
        QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
        request.setMchid(wxPayConfig.getMerchantId());
        request.setOutTradeNo(billNo);
        return jsapiService.queryOrderByOutTradeNo(request);
    }

    @Override
    public boolean isSuccess(Transaction transaction) {
        return Transaction.TradeStateEnum.SUCCESS.equals(transaction.getTradeState());
    }

    /**
     * 微信支付通知
     * @param request 请求
     */
    @Override
    @Transactional
    public void payNotify(HttpServletRequest request) {
        // 获取原始报文body
        String body = HttpUtils.getBody(request);
        log.info("获取原始报文body: {}", body);
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);
        log.info("解析通知数据: {}", notification);
        // 支付成功通知
        if (NotifyEventType.TRANSACTION_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 Transaction
            Transaction transaction = checkAndParse(request, body, Transaction.class);
            log.info("验签、解密并转换成Transaction: {}", transaction);

            if (Transaction.TradeStateEnum.SUCCESS.equals(transaction.getTradeState())) {
                // 充值成功，修改订单状态
                transactionBillService.rechargeSuccess(transaction.getOutTradeNo(), DateUtils.getNowDate());
                // 保存通知数据
//                    saveNotifyData(transaction.getOutTradeNo(), notification, transaction);
            }
        }
    }

    /**
     * 提现打款
     * @param billId id
     */
    @Override
    @Transactional
    public InitiateBatchTransferResponse payWithdraw(Long billId) {
        ServiceUtil.assertion(billId == null, "订单id不允许为空");
        TransactionBill bill = transactionBillService.selectSmTransactionBillByBillId(billId);
        ServiceUtil.assertion(bill == null, "单据不存在");
        ServiceUtil.assertion(TransactionBillStatus.WITHDRAW_PASSED.getStatus().equals(bill.getStatus()), "当前提现单据状态异常");
        ServiceUtil.assertion(StringUtils.isBlank(bill.getAccountNo()), "提现账号异常");

        // 转账明细列表
        List<TransferDetailInput> transferDetailList = this.buildTransferDetailList(bill.getArrivalAmount(), bill.getAccountNo(), TransferScene.WITHDRAW);

        // 更新微信批次明细单号
        transactionBillService.updateWxTransferDetailIds(billId, transferDetailList.stream().map(TransferDetailInput::getOutDetailNo).collect(Collectors.toList()));

        // 发起转账请求
        InitiateBatchTransferRequest request = new InitiateBatchTransferRequest();
        request.setAppid(wxPayConfig.getAppId());
        request.setOutBatchNo(bill.getBillNo());
        request.setBatchName("提现");
        request.setBatchRemark(String.format("用户%s提现%f元", bill.getUserId(), bill.getArrivalAmount()));
        request.setTotalAmount(getTransferAmount(bill.getArrivalAmount()));
        request.setTotalNum(transferDetailList.size());
        request.setTransferDetailList(transferDetailList);
        return transferBatchService.initiateBatchTransfer(request);
    }

    @Override
    @Transactional
    public void wxTransferNotify(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);

        // 判断是否重复通知，重复通知则忽略
        if (isRepeatNotify(notification.getId())) {
            return;
        }

        // 商户转账成功通知
        if (NotifyEventType.MCHTRANSFER_BATCH_FINISHED.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 TransferBatchGet
            TransferBatchGet transferBatchGet = checkAndParse(request, body, TransferBatchGet.class);
            TransactionBill bill = transactionBillService.selectSmTransactionBillByBillNo(transferBatchGet.getOutBatchNo());
            ServiceUtil.assertion(bill == null, "订单不存在");

            // 修改订单状态
            if (TransferBatchStatus.FINISHED.getStatus().equals(transferBatchGet.getBatchStatus())) {
                // 提现成功
                transactionBillService.withdrawSuccess(bill.getBillId());
            } else if (TransferBatchStatus.CLOSED.getStatus().equals(transferBatchGet.getBatchStatus())) {
                // 提现失败
                transactionBillService.withdrawFailed(bill.getBillId());
            }
        }


    }

    @Override
    public List<TransferDetailInput> buildTransferDetailList(BigDecimal totalAmount, String openId, TransferScene transferScene) {
        ServiceUtil.assertion(BigDecimal.ZERO.compareTo(totalAmount) > 0, "转账总金额不允许小于0");
        ServiceUtil.assertion(StringUtils.isBlank(openId), "微信openId不允许为空");
        ServiceUtil.assertion(transferScene == null, "转账场景不允许为空");

        List<TransferDetailInput> transferDetailList = new ArrayList<>();
        BigDecimal limitAmount = transferScene.getLimitAmount();   // 单笔限额200元
        BigDecimal payAmount = totalAmount;   // 需要转账的金额
        while (BigDecimal.ZERO.compareTo(payAmount) < 0) {
            BigDecimal detailPayAmount = payAmount.compareTo(limitAmount) > 0 ? limitAmount : payAmount;    // 当前明细转账的金额
            TransferDetailInput input = new TransferDetailInput();
            input.setOutDetailNo(String.valueOf(SnowFlakeUtil.newId()));
            input.setTransferAmount(getTransferAmount(detailPayAmount));
            input.setTransferRemark(transferScene.getRemark());
            input.setOpenid(openId);
            transferDetailList.add(input);
            payAmount = payAmount.subtract(detailPayAmount);
        }
        return transferDetailList;
    }

    /**
     * 发起退款
     *
     * @param refund
     */
    @Override
    public Refund refund(RefundAble refund) {
        CreateRequest request = new CreateRequest();
        request.setOutTradeNo(refund.refundOutTradeNo());
        request.setOutRefundNo(refund.refundOutRefundNo());
        request.setReason(refund.refundReason());
        request.setAmount(refund.refundAmount());
        request.setNotifyUrl(wxPayConfig.getRefundNotifyUrl());
        log.info("【退款】请求微信参数: {}", JSON.toJSONString(request));
        Refund res = refundService.create(request);
        log.info("【退款】微信返回结果:【{}】",JSON.toJSONString(refund));
        return res;
    }

    private Long getTransferAmount(BigDecimal money) {
        return money.multiply(new BigDecimal(100)).longValue();
    }

    /**
     * 通知是否重复
     * @param notifyId 通知id
     */
    private boolean isRepeatNotify(String notifyId) {
        SmWxPayNotify repeat = smWxPayNotifyMapper.selectSmWxPayNotifyByNotifyId(notifyId);
        return repeat != null;
    }

    /**
     * 保存通知数据
     * @param billNo 订单编号
     * @param body  请求体
     * @param plaintext 明文
     */
    private void saveNotifyData(String billNo, Notification body, Object plaintext) {
        TransactionBill bill = transactionBillService.selectSmTransactionBillByBillNo(billNo);
        SmWxPayNotify data = new SmWxPayNotify();
        data.setNotifyId(body.getId());
        data.setBillId(bill.getBillId());
        data.setBody(JSON.toJSONString(body));
        data.setPlaintext(JSON.toJSONString(plaintext));
        int i = smWxPayNotifyMapper.insertSmWxPayNotify(data);
        ServiceUtil.assertion(i != 1, "保存通知数据失败");
    }

    /**
     * 验签并解析
     * @param request 请求
     * @param body 请求体
     * @param clazz 返回值类型
     */
    @Override
    public <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
        // 构造 RequestParam
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(request.getHeader("Wechatpay-Serial"))
                .nonce(request.getHeader("Wechatpay-Nonce"))
                .signature(request.getHeader("Wechatpay-Signature"))
                .timestamp(request.getHeader("Wechatpay-Timestamp"))
                .body(body)
                .build();
        // 验签
        return notificationParser.parse(requestParam, clazz);
    }

    private Payer getPayer(String openId) {
        Payer payer = new Payer();
        payer.setOpenid(openId);
        return payer;
    }

    private Amount getAmount(BigDecimal money) {
        Amount amount = new Amount();
        amount.setTotal(money.multiply(new BigDecimal(100)).intValue());
        amount.setCurrency(CNY);
        return amount;
    }

    private String getDescription(TransactionBill bill) {
        return TransactionBillType.parse(bill.getType()).getDescription();
    }

}
